package forum

import (
	"net/http"
	"regexp"
	"strconv"

	"git.kirsle.net/apps/gosocial/pkg/config"
	"git.kirsle.net/apps/gosocial/pkg/log"
	"git.kirsle.net/apps/gosocial/pkg/models"
	"git.kirsle.net/apps/gosocial/pkg/session"
	"git.kirsle.net/apps/gosocial/pkg/templates"
)

var ThreadPathRegexp = regexp.MustCompile(`^/forum/thread/(\d+)$`)

// Thread view for a specific board index.
func Thread() http.HandlerFunc {
	tmpl := templates.Must("forum/thread.html")
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		// Parse the path parameters
		var (
			forum  *models.Forum
			thread *models.Thread
		)

		if m := ThreadPathRegexp.FindStringSubmatch(r.URL.Path); m == nil {
			log.Error("Regexp failed to parse: %s", r.URL.Path)
			templates.NotFoundPage(w, r)
			return
		} else {
			if threadID, err := strconv.Atoi(m[1]); err != nil {
				session.FlashError(w, r, "Invalid thread ID in the address bar.")
				templates.Redirect(w, "/forum")
				return
			} else {
				// Load the thread.
				if found, err := models.GetThread(uint64(threadID)); err != nil {
					session.FlashError(w, r, "That thread does not exist.")
					templates.Redirect(w, "/forum")
					return
				} else {
					thread = found
					forum = &thread.Forum
				}
			}
		}

		// Get the current user.
		currentUser, err := session.CurrentUser(r)
		if err != nil {
			session.FlashError(w, r, "Couldn't get current user: %s", err)
			templates.Redirect(w, "/")
			return
		}

		// Ping the view count on this thread.
		if err := thread.View(); err != nil {
			log.Error("Couldn't ping view count on thread %d: %s", thread.ID, err)
		}

		// Paginate the comments on this thread.
		var pager = &models.Pagination{
			Page:    1,
			PerPage: config.PageSizeThreadList,
			Sort:    "created_at asc",
		}
		pager.ParsePage(r)

		comments, err := models.PaginateComments(currentUser, "threads", thread.ID, pager)
		if err != nil {
			session.FlashError(w, r, "Couldn't paginate comments: %s", err)
			templates.Redirect(w, "/")
			return
		}

		var vars = map[string]interface{}{
			"Forum":    forum,
			"Thread":   thread,
			"Comments": comments,
			"Pager":    pager,
		}
		if err := tmpl.Execute(w, r, vars); err != nil {
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}
	})
}
